---------------------------------------------------------------------------
--- capi: awesome C API
--
-- @author Julien Danjou &lt;julien@danjou.info&gt;
-- @copyright 2008 Julien Danjou
---------------------------------------------------------------------------
module("capi")
--- Client object.
-- @field id The window X id.
-- @field name The client title.
-- @field skip_taskbar True if the client does not want to be in taskbar.
-- @field type The window type (desktop, normal, dock, …).
-- @field class The client class.
-- @field instance The client instance.
-- @field pid The client PID, if available.
-- @field role The window role, if available.
-- @field machine The machine client is running on.
-- @field icon_name The client name when iconified.
-- @field screen Client screen number.
-- @field hide Define if the client must be hidden, i.e. never mapped,
-- invisible in taskbar.
-- @field minimized Define it the client must be iconify, i.e. only visible in
-- taskbar.
-- @field size_hints_honor Honor size hints, i.e. respect size ratio.
-- @field border_width The client border width.
-- @field border_color The client border color.
-- @field titlebar The client titlebar.
-- @field urgent The client urgent state.
-- @field content An image representing the client window content (screenshot).
-- @field focus The focused client.
-- @field opacity The client opacity between 0 and 1.
-- @field ontop The client is on top of every other windows.
-- @field above The client is above normal windows.
-- @field below The client is below normal windows.
-- @field fullscreen The client is fullscreen or not.
-- @field maximized_horizontal The client is maximized horizontally or not.
-- @field maximized_vertical The client is maximized vertically or not.
-- @field transient_for Return the client the window is transient for.
-- @field group_id Identification unique to a group of windows.
-- @field leader_id Identification unique to windows spawned by the same command.
-- @field size_hints A table with size hints of the client: user_position,
--         user_size, program_position, program_size, etc.
-- @class table
-- @name client
client
--- Get all clients into a table.
-- @param arg1 An optional screen nunmber.
-- @return A table with all clients.
function client.get()
end

--- Check if a client is visible on its screen.
-- @return A boolean value, true if the client is visible, false otherwise.
function client.isvisible()
end

--- Return client geometry.
-- @param arg1 A table with new coordinates, or none.
-- @return A table with client coordinates.
function client.geometry()
end

--- Return client struts (reserved space at the edge of the screen).
-- @param arg1 A table with new strut values, or none.
-- @return A table with strut values.
function client.struts()
end

--- Get or set mouse buttons bindings for a client.
-- @param arg1 An array of mouse button bindings objects, or nothing.
function client.buttons()
end

--- Get or set keys bindings for a client.
-- @param arg1 An array of key bindings objects, or nothing.
function client.keys()
end

--- Access or set the client tags.
-- @param arg1 A table with tags to set, or none to get the current tags table.
function client.tags()
end

--- Kill a client.
--
function client.kill()
end

--- Swap a client with another one.
-- @param arg1 A client to swap with.
function client.swap()
end

--- Raise a client on top of others which are on the same layer.
function client.raise()
end

--- Lower a client on bottom of others which are on the same layer.
function client.lower()
end

--- Redraw a client by unmapping and mapping it quickly.
--
function client.redraw()
end

--- Stop managing a client.
function client.unmanage()
end

--- Register a D-Bus name to receive message from.
-- @param arg1 A string indicating if we are using system or session bus.
-- @param arg2 A string with the name of the D-Bus name to register. Note that
-- org.awesome is registered by default.
-- @return True if everything worked fine, false otherwise.
function dbus.request_name()
end

--- Release a D-Bus name.
-- @param arg1 A string indicating if we are using system or session bus.
-- @param arg2 A string with the name of the D-Bus name to unregister.
-- @return True if everything worked fine, false otherwise.
function dbus.release_name()
end

--- Add a match rule to match messages going through the message bus.
-- @param arg1 A string indicating if we are using system or session bus.
-- @param arg2 A string with the name of the match rule.
function dbus.add_match()
end

--- Remove a previously added match rule "by value"
-- (the most recently-added identical rule gets removed).
-- @param arg1 A string indicating if we are using system or session bus.
-- @param arg2 A string with the name of the match rule.
function dbus.remove_match()
end

--- Get or set global mouse bindings.
-- This binding will be available when you'll click on root window.
-- @param arg1 An array of mouse button bindings objects, or nothing.
-- @return The array of mouse button bindings objects.
function root.buttons()
end

--- Get or set global key bindings.
-- This binding will be available when you'll press keys on root window.
-- @param arg1 An array of key bindings objects, or nothing.
-- @return The array of key bindings objects of this client.
function root.keys()
end

--- Set the root cursor.
-- @param arg1 A X cursor name.
function root.cursor()
end

--- Send fake events. Usually the current focused client will get it.
-- @param arg1 The event type: key_press, key_release, button_press, button_release
-- or motion_notify.
-- @param arg2 The detail: in case of a key event, this is the keycode to send, in
-- case of a button event this is the number of the button. In case of a motion
-- event, this is a boolean value which if true make the coordinates relatives.
-- @param arg3 In case of a motion event, this is the X coordinate.
-- @param arg4 In case of a motion event, this is the Y coordinate.
-- @param arg5 In case of a motion event, this is the screen number to move on.
-- If not specified, the current one is used.
function root.fake_input()
end

--- Key object.
-- @field key The key to press to triggers an event.
-- @field keysym Same as key, but return the name of the key symbol. It can
-- be identical to key, but for characters like '.' it will return 'period'.
-- @field modifiers The modifier key that should be pressed while the key is
-- pressed. An array with all the modifiers. Valid modifiers are: Any, Mod1,
-- Mod2, Mod3, Mod4, Mod5, Shift, Lock and Control.
-- @field press The function which is called when the key combination is pressed.
-- @field release The function which is called when the key combination is released.
-- @class table
-- @name key
key
--- Define a global key binding. This key binding will always be available.
--
-- @param arg1 A table with modifier keys: can be Control or Ctrl, Shift, Lock,
-- Mod1, Mod2, Mod3, Mod4, Mod5 or Any.
-- @param arg2 A key name.
-- @param arg3 A function to execute on key press.
-- @param arg4 A function to execute on key release.
-- @return The key.
function key()
end

--- Grab keyboard and read pressed keys, calling callback function at each key
-- pressed. The callback function must return a boolean value: true to
-- continue grabbing, false to stop.
-- The function is called with 3 arguments:
-- a table containing modifiers keys, a string with the key pressed and a
-- string with eithe "press" or "release" to indicate the event type.
--
--
--
-- @param arg1 A callback function as described above.
function keygrabber.run()
end

--- Stop grabbing the keyboard.
function keygrabber.stop()
end

--- Grab the mouse pointer and list motions, calling callback function at each
-- motion. The callback function must return a boolean value: true to
-- continue grabbing, false to stop.
-- The function is called with one argument:
-- a table containing modifiers pointer coordinates.
--
--
--
-- @param arg1 A callback function as described above.
function mousegrabber.run()
end

--- Stop grabbing the mouse pointer.
function mousegrabber.stop()
end

--- Index function of wtable objects.
-- @class table
-- @name wtable
wtable
--- awesome global table.
-- @field font The default font.
-- @field conffile The configuration file which has been loaded.
-- @class table
-- @name awesome
awesome
--- Quit awesome.
function awesome.quit()
end

--- Execute another application, probably a window manager, to replace
-- awesome.
-- @param arg1 The command line to execute.
function awesome.exec()
end

This function is not yet documented.

--- Restart awesome.
function awesome.restart()
end

--- Set the function called each time a client gets focus. This function is
-- called with the client object as argument.
-- @param arg1 A function to call each time a client gets focus.
function hooks.focus()
end

--- Set the function called each time a client loses focus. This function is
-- called with the client object as argument.
-- @param arg1 A function to call each time a client loses focus.
function hooks.unfocus()
end

--- Set the function called each time a new client appears. This function is
-- called with the client object as argument, plus a boolean argument which is
-- true if the client is managed at startup.
-- @param arg1 A function to call on each new client.
function hooks.manage()
end

--- Set the function called each time a client goes away. This function is
-- called with the client object as argument.
-- @param arg1 A function to call when a client goes away.
function hooks.unmanage()
end

--- Set the function called each time the mouse enter a window. This
-- function is called with the client object as argument.
-- @param arg1 A function to call each time a client gets mouse over it.
function hooks.mouse_enter()
end

--- Set the function called each time the mouse leave a window. This
-- function is called with the client object as argument.
-- @param arg1 A function to call each time a client gets mouse over it.
function hooks.mouse_leave()
end

--- Set the function called on each client's property change.
-- This function is called with the client object as argument and the
-- property name.
-- @param arg1 A function to call on each client property update.
function hooks.property()
end

--- Set the function called on each screen arrange. This function is called
-- with the screen number as argument.
-- @param arg1 A function to call on each screen arrange.
function hooks.arrange()
end

--- Set the function called on each client list change.
-- This function is called without any argument.
-- @param arg1 A function to call on each client list change.
function hooks.clients()
end

--- Set the function called on each screen tag list change.
-- This function is called with a screen number as first argument,
-- the tag object as second and the action (add or remove) as third.
-- @param arg1 A function to call on each tag list change.
function hooks.tags()
end

--- Set the function called on each client's tags change.
-- This function is called with the client and the tag as argument.
-- @param arg1 A function to call on each client's tags change.
function hooks.tagged()
end

--- Set the function called on each startup-notification events
-- This function is called with a table and various fields set to describe the
-- vents.
-- @param arg1 A function to call on each startup-notification event.
function hooks.startup_notification()
end

--- Set the function to be called every N seconds.
-- @param arg1 The number of seconds to run function every. Set 0 to disable.
-- @param arg2 A function to call every N seconds (optional).
function hooks.timer()
end

--- Set the function to be called when a D-Bus event is received.
-- The first argument passed to this function is the type of the message we
-- receive: signal, method_call, method_return or error.
-- The second argument is the path.
-- The other arguments are a variable list of arguments.
-- The function can return values using pair of type, value.
-- For example: return "s", "hello", "i", 32
-- @param arg1 A function to call on D-Bus events.
function hooks.dbus()
end

--- Mouse library.
-- @field coords Mouse coordinates.
-- @field screen Mouse screen number.
-- @class table
-- @name mouse
mouse
--- Get or set the mouse coords.
-- @param arg1 None or a table with x and y keys as mouse coordinates.
-- @return A table with mouse coordinates.
function mouse.coords()
end

--- Get the client which is under the pointer.
-- @return A client or nil.
function mouse.object_under_pointer()
end

--- Button object.
-- @field press The function called when button press event is received.
-- @field release The function called when button release event is received.
-- @field button The mouse button number, or 0 for any button.
-- @field modifiers The modifier key table that should be pressed while the
-- button is pressed.
-- @class table
-- @name button
button
--- Create a new mouse button bindings.
-- @param arg1 A table with modifiers keys, or a button to clone.
-- @param arg2 A mouse button number, or 0 to match any button.
-- @param arg3 A function to execute on click events.
-- @param arg4 A function to execute on release events.
-- @return A mouse button binding.
function button()
end

--- A screen.
-- @field coords The screen coordinates. Immutable.
-- @field workarea The screen workarea, i.e. without wiboxes.
-- @class table
-- @name screen
screen
--- Get the screen count.
--
-- @return The screen count, at least 1.
function screen.count()
end

--- Get or set screen tags.
-- @param arg1 None or a table of tags to set to the screen.
-- The table must contains at least one tag.
function screen.tags()
end

--- Set or get the screen padding.
-- @param arg1 None or a table with new padding values.
-- @return The screen padding. A table with top, right, left and bottom
-- keys and values in pixel.
function screen.padding()
end

--- Wibox object.
-- @field screen Screen number.
-- @field client The client attached to (titlebar).
-- @field border_width Border width.
-- @field border_color Border color.
-- @field align The alignment.
-- @field fg Foreground color.
-- @field bg Background color.
-- @field bg_image Background image.
-- @field position The position.
-- @field ontop On top of other windows.
-- @field cursor The mouse cursor.
-- @field visible Visibility.
-- @field orientation The drawing orientation: east, north or south.
-- @field widgets An array with all widgets drawn on this wibox.
-- @field opacity The opacity of the wibox, between 0 and 1.
-- @field mouse_enter A function to execute when the mouse enter the widget.
-- @field mouse_leave A function to execute when the mouse leave the widget.
-- @class table
-- @name wibox
wibox
--- Create a new wibox.
--
-- @param arg1 A table with optionaly defined values:
-- position, align, fg, bg, border_width, border_color, ontop, width and height.
-- @return A brand new wibox.
function wibox()
end

--- Get or set mouse buttons bindings to a wibox.
-- @param arg1 An array of mouse button bindings objects, or nothing.
function wibox.buttons()
end

This function is not yet documented.

--- Tag object.
-- @field name Tag name.
-- @field screen Screen number of the tag.
-- @field layout Tag layout.
-- @field selected True if the client is selected to be viewed.
-- @class table
-- @name tag
tag
--- Create a new tag.
-- @param arg1 A name.
-- @return A new tag object.
function tag()
end

--- Get or set the clients attached to this tag.
-- @param arg1 None or a table of clients to set.
-- @return A table with the clients attached to this tags.
function tag.clients()
end

--- Generic widget.
-- @field align The widget alignment.
-- @field visible The widget visibility.
-- @field mouse_enter A function to execute when the mouse enter the widget.
-- @field mouse_leave A function to execute when the mouse leave the widget.
-- @class table
-- @name widget
widget
--- Create a new widget.
--
-- @param arg1 A table with at least a type value. Optional attributes
-- are: align.
-- @return A brand new widget.
function widget()
end

--- Get or set mouse buttons bindings to a widget.
--
-- @param arg1 An array of mouse button bindings objects, or nothing.
function widget.buttons()
end

--- Image object.
-- @field width The image width.
-- @field height The image height.
-- @class table
-- @name image
image
--- Create a new image object.
-- @param arg1 The image path.
-- @return An image object.
function image()
end

--- Create a new image object from ARGB32 data.
-- @param arg1 The image width.
-- @param arg2 The image height.
-- @param arg3 The image data as a string in ARGB32 format, or nil to create an
-- empty image.
-- @return An image object.
function image.argb32()
end

--- Rotate an image with specified angle radians and return a new image.
-- @param arg1 The angle in radians.
-- @return A rotated image.
function image.rotate()
end

--- Performs 90 degree rotations on the current image. Passing 0 orientation
-- does not rotate, 1 rotates clockwise by 90 degree, 2, rotates clockwise by
-- 180 degrees, 3 rotates clockwise by 270 degrees.
-- @param arg1 The rotation to perform.
function image.orientate()
end

--- Crop an image to the given rectangle.
-- @param arg1 The top left x coordinate of the rectangle.
-- @param arg2 The top left y coordinate of the rectangle.
-- @param arg3 The width of the rectangle.
-- @param arg4 The height of the rectangle.
-- @return A cropped image.
function image.crop()
end

--- Crop the image to the given rectangle and scales it.
-- @param arg1 The top left x coordinate of the source rectangle.
-- @param arg2 The top left y coordinate of the source rectangle.
-- @param arg3 The width of the source rectangle.
-- @param arg4 The height of the source rectangle.
-- @param arg5 The width of the destination rectangle.
-- @param arg6 The height of the destination rectangle.
-- @return A cropped image.
function image.crop_and_scale()
end

--- Saves the image to the given path. The file extension (e.g. .png or .jpg)
-- will affect the output format.
-- @param arg1 The image path.
function image.save()
end

--- Draw a pixel in an image
-- \lvalua An image.
-- @param arg1 The x coordinate of the pixel to draw
-- @param arg2 The y coordinate of the pixel to draw
-- @param arg3 The color to draw the pixel in
function image.draw_pixel()
end

This function is not yet documented.

--- Draw a rectangle in an image
-- \lvalua An image.
-- @param arg1 The x coordinate of the rectangles top left corner
-- @param arg2 The y coordinate of the rectangles top left corner
-- @param arg3 The width of the rectangle
-- @param arg4 The height of the rectangle
-- @param arg5 True if the rectangle should be filled, False otherwise
-- @param arg6 The color to draw the rectangle in
function image.draw_rectangle()
end

--- Draw a circle in an image
-- \lvalua An image.
-- @param arg1 The x coordinate of the center of the circle
-- @param arg2 The y coordinate of the center of the circle
-- @param arg3 The horizontal amplitude (width)
-- @param arg4 The vertical amplitude (height)
-- @param arg5 True if the circle should be filled, False otherwise
-- @param arg6 The color to draw the circle in
function image.draw_circle()
end

--- Graph widget.
-- @field plot_properties_set A function to set plot properties.
-- @field plot_data_add A function to add data to a plot.
-- @field height Graph height.
-- @field widget Graph width.
-- @field bg Background color.
-- @field grow Direction to grow: left or right.
-- @class table
-- @name graph
graph
--- Set various plot graph properties.
-- @param arg1 A plot name.
-- @param arg2 A table with various properties set.
function graph.plot_properties_set()
end

--- Add data to a plot.
-- @param arg1 A plot name.
-- @param arg2 A data value.
function graph.plot_data_add()
end

--- Progressbar widget.
-- @field bar_properties_set Set the properties of a bar.
-- @field bar_data_add Add data to a bar.
-- @field gap Gap betweens bars.
-- @field ticks_gap Gap between ticks.
-- @field ticks_count Number of ticks.
-- @field border_padding Border padding.
-- @field border_width Border width.
-- @field width Bars width.
-- @field height Bars height.
-- @field vertical True: draw bar vertically, false: horizontally.
-- @class table
-- @name progressbar
progressbar
--- Set various progressbar bars properties:
-- @param arg1 A bar name.
-- @param arg2 A table with keys as properties names.
function progressbar.bar_properties_set()
end

--- Add a value to a progressbar bar.
-- @param arg1 A bar name.
-- @param arg2 A data value.
function progressbar.bar_data_add()
end

--- Textbox widget.
-- @field text The text to display.
-- @field width The width of the textbox. Set to 0 for auto.
-- @field wrap The wrap mode: word, char, word_char.
-- @field ellipsize The ellipsize mode: start, middle or end.
-- @field border_width The border width to draw around.
-- @field border_color The border color.
-- @field align Text alignment, left, center or right.
-- @field margin Method to pass text margin: a table with top, left, right and bottom keys.
-- @field bg Background color.
-- @field bg_image Background image.
-- @field bg_align Background image alignment, left, center, right, bottom, top or middle
-- @field bg_resize Background resize.
-- @class table
-- @name textbox
textbox
--- Imagebox widget.
-- @field image The image to display.
-- @field bg The background color to use.
-- @class table
-- @name imagebox
imagebox
